package com.minemaarten.signals.client.render.signals;

import gnu.trove.map.hash.TIntObjectHashMap;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.Vec3d;

import org.lwjgl.opengl.GL11;

import com.google.common.collect.ImmutableList;
import com.minemaarten.signals.client.BakedRenderer;
import com.minemaarten.signals.lib.HeadingUtils;
import com.minemaarten.signals.rail.network.EnumHeading;
import com.minemaarten.signals.rail.network.NetworkRail;
import com.minemaarten.signals.rail.network.RailEdge;
import com.minemaarten.signals.rail.network.mc.MCNetworkRail;
import com.minemaarten.signals.rail.network.mc.MCPos;
import com.minemaarten.signals.rail.network.mc.RailNetworkManager;

public class DirectionalityRenderer{

    private TIntObjectHashMap<BakedRenderer> bakedRenderers = new TIntObjectHashMap<>();

    public void render(int dimensionID, BufferBuilder b){
        BakedRenderer bakedRenderer = bakedRenderers.get(dimensionID);
        if(bakedRenderer != null) {
            b.func_181668_a(GL11.GL_TRIANGLES, DefaultVertexFormats.field_181705_e);
            bakedRenderer.render(b);
            Tessellator.func_178181_a().func_78381_a();
        }
    }

    public void updateRender(){
        TIntObjectHashMap<BakedRenderer> bakedRenderers = new TIntObjectHashMap<>();
        for(RailEdge<MCPos> edge : RailNetworkManager.getServerInstance().getNetwork().getAllEdges()) {
            if(edge.directionality.canTravelForwards) build(bakedRenderers, edge.edge);
            if(edge.directionality.canTravelBackwards) build(bakedRenderers, edge.edge.reverse());
        }
        this.bakedRenderers = bakedRenderers;
    }

    private void build(TIntObjectHashMap<BakedRenderer> bakedRenderers, ImmutableList<NetworkRail<MCPos>> edge){

        for(int edgeIndex = 1; edgeIndex < edge.size() - 1; edgeIndex++) {
            NetworkRail<MCPos> prevRail = edge.get(edgeIndex - 1);
            MCNetworkRail curRail = (MCNetworkRail)edge.get(edgeIndex);
            NetworkRail<MCPos> nextRail = edge.get(edgeIndex + 1);

            EnumHeading prevHeading = curRail.pos.getRelativeHeading(prevRail.pos);
            EnumHeading nextHeading = nextRail.pos.getRelativeHeading(curRail.pos);
            if(prevHeading == null || nextHeading == null || prevHeading != nextHeading || curRail.pos.getDimID() != nextRail.pos.getDimID() || curRail.pos.getDimID() != prevRail.pos.getDimID()) continue;

            BakedRenderer bakedRenderer = bakedRenderers.get(curRail.pos.getDimID());
            if(bakedRenderer == null) {
                bakedRenderer = new BakedRenderer();
                bakedRenderers.put(curRail.pos.getDimID(), bakedRenderer);
            }

            MCPos pos = curRail.pos;
            EnumFacing facing = HeadingUtils.toFacing(nextHeading).func_176734_d();
            EnumFacing rotatedFacing = facing.func_176746_e();
            EnumFacing rotatedFacing2 = facing.func_176735_f();
            int yOffset = AbstractRailRenderer.getRailHeightOffset(curRail, facing);

            Vec3d posVec = new Vec3d(pos.getX() + 0.5, pos.getY() + (yOffset != 0 ? 0.6001 : 0.1001), pos.getZ() + 0.5);

            double arrowSize = 0.1;
            double spacing = 0.1;

            for(int i = -2; i < -1; i++) {
                Vec3d shiftedPosVec = posVec.func_72441_c(facing.func_82601_c() * spacing * i, spacing * i * yOffset + 0.001, facing.func_82599_e() * spacing * i);
                Vec3d vecBack = shiftedPosVec.func_72441_c(facing.func_82601_c() * arrowSize, arrowSize * yOffset, facing.func_82599_e() * arrowSize);
                Vec3d c1 = vecBack.func_72441_c(rotatedFacing.func_82601_c() * arrowSize, 0, rotatedFacing.func_82599_e() * arrowSize);
                Vec3d c2 = vecBack.func_72441_c(rotatedFacing2.func_82601_c() * arrowSize, 0, rotatedFacing2.func_82599_e() * arrowSize);

                bakedRenderer.add(shiftedPosVec.field_72450_a, shiftedPosVec.field_72448_b, shiftedPosVec.field_72449_c);
                bakedRenderer.add(c1.field_72450_a, c1.field_72448_b, c1.field_72449_c);
                //buffer.pos(shiftedPosVec.x, shiftedPosVec.y, shiftedPosVec.z).color(r, g, b, 1).endVertex();
                bakedRenderer.add(c2.field_72450_a, c2.field_72448_b, c2.field_72449_c);
            }
        }
    }
}
