package com.minemaarten.signals.inventory;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IContainerListener;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;

import com.minemaarten.signals.client.gui.GuiContainerBase;
import com.minemaarten.signals.inventory.slots.IPhantomSlot;
import com.minemaarten.signals.network.GuiSynced;
import com.minemaarten.signals.network.NetworkHandler;
import com.minemaarten.signals.network.NetworkUtils;
import com.minemaarten.signals.network.PacketUpdateGui;
import com.minemaarten.signals.network.SyncedField;
import com.minemaarten.signals.tileentity.IGUIButtonSensitive;

public class ContainerBase<Tile extends TileEntity> extends Container implements IGUIButtonSensitive{

    public Tile te;
    private final List<SyncedField> syncedFields = new ArrayList<>();
    private boolean firstTick = true;
    private int playerSlotsStart;

    public ContainerBase(Tile te){

        this.te = te;
        if(te != null) addSyncedFields(te);
    }

    protected void addSyncedField(SyncedField field){

        syncedFields.add(field);
        field.setLazy(false);
    }

    protected void addSyncedFields(Object annotatedObject){

        List<SyncedField> fields = NetworkUtils.getSyncedFields(annotatedObject, GuiSynced.class);
        for(SyncedField field : fields)
            addSyncedField(field);
    }

    public void updateField(int index, Object value){

        syncedFields.get(index).setValue(value);
        if(Minecraft.func_71410_x().field_71462_r instanceof GuiContainerBase) {
            ((GuiContainerBase<?>)Minecraft.func_71410_x().field_71462_r).onFieldSynced();
        }
    }

    @Override
    public boolean func_75145_c(EntityPlayer player){

        return te == null ? false : player.func_174791_d().func_72438_d(new Vec3d(te.func_174877_v())) < 8;
    }

    @Override
    public void func_75142_b(){

        super.func_75142_b();
        for(int i = 0; i < syncedFields.size(); i++) {
            if(syncedFields.get(i).update() || firstTick) {
                sendToCrafters(new PacketUpdateGui(i, syncedFields.get(i)));
            }
        }
        firstTick = false;
    }

    protected void sendToCrafters(IMessage message){

        for(IContainerListener crafter : field_75149_d) {
            if(crafter instanceof EntityPlayerMP) {
                NetworkHandler.sendTo(message, (EntityPlayerMP)crafter);
            }
        }
    }

    protected void addPlayerSlots(InventoryPlayer inventoryPlayer, int yOffset){
        addPlayerSlots(inventoryPlayer, 8, yOffset);
    }

    protected void addPlayerSlots(InventoryPlayer inventoryPlayer, int xOffset, int yOffset){

        playerSlotsStart = field_75151_b.size();

        // Add the player's inventory slots to the container
        for(int inventoryRowIndex = 0; inventoryRowIndex < 3; ++inventoryRowIndex) {
            for(int inventoryColumnIndex = 0; inventoryColumnIndex < 9; ++inventoryColumnIndex) {
                func_75146_a(new Slot(inventoryPlayer, inventoryColumnIndex + inventoryRowIndex * 9 + 9, xOffset + inventoryColumnIndex * 18, yOffset + inventoryRowIndex * 18));
            }
        }

        // Add the player's action bar slots to the container
        for(int actionBarSlotIndex = 0; actionBarSlotIndex < 9; ++actionBarSlotIndex) {
            func_75146_a(new Slot(inventoryPlayer, actionBarSlotIndex, xOffset + actionBarSlotIndex * 18, yOffset + 58));
        }
    }

    @Override
    public ItemStack func_82846_b(EntityPlayer par1EntityPlayer, int par2){

        ItemStack var3 = ItemStack.field_190927_a;
        Slot var4 = field_75151_b.get(par2);

        if(var4 != null && var4.func_75216_d()) {
            ItemStack var5 = var4.func_75211_c();
            var3 = var5.func_77946_l();

            if(par2 < playerSlotsStart) {
                if(!func_75135_a(var5, playerSlotsStart, playerSlotsStart + 36, false)) return ItemStack.field_190927_a;

                var4.func_75220_a(var5, var3);
            } else {
                if(!func_75135_a(var5, 0, playerSlotsStart, false)) return ItemStack.field_190927_a;
                var4.func_75220_a(var5, var3);
            }

            if(var5.func_190926_b()) {
                var4.func_75215_d(ItemStack.field_190927_a);
            } else {
                var4.func_75218_e();
            }

            if(var5.func_190916_E() == var3.func_190916_E()) return ItemStack.field_190927_a;

            var4.func_190901_a(par1EntityPlayer, var5);
        }

        return var3;
    }

    /**
     * Source: Buildcraft
     */
    /* @Override
     public ItemStack slotClick(int slotNum, int modifier, int mouseButton, EntityPlayer player){

         Slot slot = slotNum < 0 ? null : (Slot)inventorySlots.get(slotNum);
         if(slot instanceof IPhantomSlot) {
             return slotClickPhantom(slot, modifier, mouseButton, player);
         }
         return super.slotClick(slotNum, modifier, mouseButton, player);
     }
    */
    private ItemStack slotClickPhantom(Slot slot, int mouseButton, int modifier, EntityPlayer player){

        ItemStack stack = ItemStack.field_190927_a;

        if(mouseButton == 2) {
            if(((IPhantomSlot)slot).canAdjust()) {
                slot.func_75215_d(ItemStack.field_190927_a);
            }
        } else if(mouseButton == 0 || mouseButton == 1) {
            InventoryPlayer playerInv = player.field_71071_by;
            slot.func_75218_e();
            ItemStack stackSlot = slot.func_75211_c();
            ItemStack stackHeld = playerInv.func_70445_o();

            if(!stackSlot.func_190926_b()) {
                stack = stackSlot.func_77946_l();
            }

            if(stackSlot.func_190926_b()) {
                if(!stackHeld.func_190926_b() && slot.func_75214_a(stackHeld)) {
                    fillPhantomSlot(slot, stackHeld, mouseButton, modifier);
                }
            } else if(stackHeld.func_190926_b()) {
                adjustPhantomSlot(slot, mouseButton, modifier);
                slot.func_190901_a(player, playerInv.func_70445_o());
            } else if(slot.func_75214_a(stackHeld)) {
                if(canStacksMerge(stackSlot, stackHeld)) {
                    adjustPhantomSlot(slot, mouseButton, modifier);
                } else {
                    fillPhantomSlot(slot, stackHeld, mouseButton, modifier);
                }
            }
        }
        return stack;
    }

    public boolean canStacksMerge(ItemStack stack1, ItemStack stack2){

        if(stack1.func_190926_b() || stack2.func_190926_b()) return false;
        if(!stack1.func_77969_a(stack2)) return false;
        if(!ItemStack.func_77970_a(stack1, stack2)) return false;
        return true;

    }

    protected void adjustPhantomSlot(Slot slot, int mouseButton, int modifier){

        if(!((IPhantomSlot)slot).canAdjust()) {
            return;
        }
        ItemStack stackSlot = slot.func_75211_c();
        int stackSize;
        if(modifier == 1) {
            stackSize = mouseButton == 0 ? (stackSlot.func_190916_E() + 1) / 2 : stackSlot.func_190916_E() * 2;
        } else {
            stackSize = mouseButton == 0 ? stackSlot.func_190916_E() - 1 : stackSlot.func_190916_E() + 1;
        }

        if(stackSize > slot.func_75219_a()) {
            stackSize = slot.func_75219_a();
        }

        stackSlot.func_190920_e(stackSize);
    }

    protected void fillPhantomSlot(Slot slot, ItemStack stackHeld, int mouseButton, int modifier){

        if(!((IPhantomSlot)slot).canAdjust()) {
            return;
        }
        int stackSize = mouseButton == 0 ? stackHeld.func_190916_E() : 1;
        if(stackSize > slot.func_75219_a()) {
            stackSize = slot.func_75219_a();
        }
        ItemStack phantomStack = stackHeld.func_77946_l();
        phantomStack.func_190920_e(stackSize);

        slot.func_75215_d(phantomStack);
    }

    @Override
    public void handleGUIButtonPress(EntityPlayer player, int... data){
        if(te instanceof IGUIButtonSensitive) ((IGUIButtonSensitive)te).handleGUIButtonPress(player, data);
    }
}
