package com.minemaarten.signals.tileentity;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.annotation.Nullable;

import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.items.CapabilityItemHandler;

import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

import com.google.common.collect.Lists;
import com.minemaarten.signals.api.ICartHopperBehaviour;
import com.minemaarten.signals.api.access.ICartHopper;
import com.minemaarten.signals.capabilities.CapabilityMinecartDestination;
import com.minemaarten.signals.init.ModBlocks;
import com.minemaarten.signals.network.GuiSynced;
import com.minemaarten.signals.rail.RailManager;
import com.minemaarten.signals.rail.network.mc.RailNetworkManager;
import com.minemaarten.signals.tileentity.carthopperbehaviour.CartHopperBehaviourItems;

import com.minemaarten.signals.api.access.ICartHopper.HopperMode;

public class TileEntityCartHopper extends TileEntityBase implements ITickable, IGUIButtonSensitive, ICartHopper{

    @GuiSynced
    private HopperMode hopperMode = HopperMode.CART_FULL;
    @GuiSynced
    private boolean interactEngine; //When true, the Cart Engine capability will be filled/emptied instead.
    private EnumFacing pushDir = EnumFacing.NORTH; //The direction the cart is pushed in when activated.
    private EntityMinecart managingCart;
    private UUID managingCartId;
    private boolean pushedLastTick;
    private int lastComparatorInputOverride;

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag){
        tag.func_74774_a("hopperMode", (byte)hopperMode.ordinal());
        tag.func_74757_a("interactEngine", interactEngine);
        tag.func_74774_a("pushDir", (byte)pushDir.ordinal());
        tag.func_74757_a("pushedLastTick", pushedLastTick);
        return super.func_189515_b(tag);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        hopperMode = HopperMode.values()[tag.func_74771_c("hopperMode")];
        interactEngine = tag.func_74767_n("interactEngine");
        pushDir = EnumFacing.field_82609_l[tag.func_74771_c("pushDir")];
        pushedLastTick = tag.func_74767_n("pushedLastTick");
    }

    @Override
    public void func_73660_a(){
        if(!func_145831_w().field_72995_K) {
            if(managingCartId != null) {
                List<EntityMinecart> carts = func_145831_w().func_175644_a(EntityMinecart.class, input -> input.getPersistentID().equals(managingCartId));
                managingCart = carts.isEmpty() ? null : carts.get(0);
                managingCartId = null;
            }

            boolean extract = RailNetworkManager.getInstance(field_145850_b.field_72995_K).getRail(func_145831_w(), func_174877_v().func_177984_a()) != null;
            updateManagingCart(new AxisAlignedBB(extract ? func_174877_v().func_177984_a() : func_174877_v().func_177977_b()));

            boolean shouldPush;
            if(managingCart != null) {
                if(isDisabled()) {
                    shouldPush = true;
                } else {
                    shouldPush = tryTransfer(extract);
                }
            } else {
                shouldPush = false;
            }
            if(shouldPush && !pushedLastTick) pushCart();
            boolean notifyNeighbors = shouldPush != pushedLastTick;
            pushedLastTick = shouldPush;
            if(notifyNeighbors) {
                func_145831_w().func_175685_c(func_174877_v(), func_145838_q(), true);
            }
            int comparatorInputOverride = getComparatorInputOverride();
            if(lastComparatorInputOverride != comparatorInputOverride) {
                field_145850_b.func_175666_e(field_174879_c, ModBlocks.CART_HOPPER);
                lastComparatorInputOverride = comparatorInputOverride;
            }
        }
    }

    private boolean isDisabled(){
        for(EnumFacing facing : EnumFacing.field_176754_o) {
            if(func_145831_w().func_175651_c(field_174879_c.func_177972_a(facing), facing) > 0) return true;
        }
        return false;
    }

    public boolean emitsRedstone(){
        return pushedLastTick;
    }

    @Override
    public HopperMode getHopperMode(){
        return hopperMode;
    }

    @Override
    public void setHopperMode(HopperMode hopperMode){
        Validate.notNull(hopperMode);
        this.hopperMode = hopperMode;
    }

    @Override
    public boolean isInteractingWithEngine(){
        return interactEngine;
    }

    @Override
    public void setInteractingWithEngine(boolean interactWithEngine){
        this.interactEngine = interactWithEngine;
    }

    private void pushCart(){
        managingCart.field_70159_w += pushDir.func_82601_c() * 0.1;
        managingCart.field_70179_y += pushDir.func_82599_e() * 0.1;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    private boolean tryTransfer(boolean extract){
        boolean active = false, empty = false, full = false;
        List<Pair<TileEntity, EnumFacing>> filters = Lists.newArrayList();
        for(EnumFacing dir : EnumFacing.field_176754_o) {
            TileEntity filter = func_145831_w().func_175625_s(func_174877_v().func_177972_a(dir));
            if(filter != null) filters.add(new ImmutablePair<>(filter, dir));
        }

        for(ICartHopperBehaviour hopperBehaviour : getApplicableHopperBehaviours(managingCart)) {
            Capability<?> cap = hopperBehaviour.getCapability();
            Object cart = null;
            if(interactEngine && hopperBehaviour instanceof CartHopperBehaviourItems) {
                if(managingCart.hasCapability(CapabilityMinecartDestination.INSTANCE, null)) {
                    cart = managingCart.getCapability(CapabilityMinecartDestination.INSTANCE, null).getEngineItemHandler();
                } else {
                    continue;
                }
            } else {
                cart = managingCart.getCapability(cap, null);
            }
            Object te = getCapabilityAt(cap, extract ? EnumFacing.DOWN : EnumFacing.UP);
            if(te != null && hopperBehaviour.tryTransfer(extract ? cart : te, extract ? te : cart, filters)) active = true;
            if(hopperMode == HopperMode.CART_EMPTY && hopperBehaviour.isCartEmpty(cart, filters)) empty = true;
            if(hopperMode == HopperMode.CART_FULL && hopperBehaviour.isCartFull(cart)) full = true;
        }
        return hopperMode == HopperMode.NO_ACTIVITY ? !active : empty || full;
    }

    private List<ICartHopperBehaviour<?>> getApplicableHopperBehaviours(EntityMinecart cart){
        Stream<ICartHopperBehaviour<?>> behaviours = RailManager.getInstance().getHopperBehaviours().stream();
        behaviours = behaviours.filter(hopperBehaviour -> interactEngine && hopperBehaviour instanceof CartHopperBehaviourItems || managingCart.hasCapability(hopperBehaviour.getCapability(), null));
        return behaviours.collect(Collectors.toList());
    }

    private void updateManagingCart(AxisAlignedBB aabb){
        if(managingCart != null) {
            if(managingCart.field_70128_L || !managingCart.func_174813_aQ().func_72326_a(aabb)) {
                managingCart = null;
            }
        }
        if(managingCart == null) {
            List<EntityMinecart> carts = func_145831_w().func_72872_a(EntityMinecart.class, aabb);
            if(!carts.isEmpty()) {
                managingCart = carts.get(0);
                pushDir = managingCart.func_184172_bi();
            }
        }
    }

    private <T> T getCapabilityAt(Capability<T> cap, EnumFacing dir){
        BlockPos pos = func_174877_v().func_177972_a(dir);
        TileEntity te = func_145831_w().func_175625_s(pos);
        return te != null && te.hasCapability(cap, dir.func_176734_d()) ? te.getCapability(cap, dir.func_176734_d()) : null;
    }

    @Override
    public void handleGUIButtonPress(EntityPlayer player, int... data){
        switch(data[0]){
            case 0:
                hopperMode = HopperMode.values()[(hopperMode.ordinal() + 1) % HopperMode.values().length];
                break;
            case 1:
                interactEngine = !interactEngine;
                break;
        }
    }

    @Override
    public boolean hasCapability(Capability<?> capability, @Nullable EnumFacing facing){
        if(capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) {
            return getCapability(capability, facing) != null;
        }
        return super.hasCapability(capability, facing);
    }

    @SuppressWarnings("unchecked")
    @Override
    @Nullable
    public <T> T getCapability(Capability<T> capability, @Nullable EnumFacing facing){
        if(managingCart != null && capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) {
            if(interactEngine) {
                CapabilityMinecartDestination destCap = managingCart.getCapability(CapabilityMinecartDestination.INSTANCE, null);
                if(destCap != null) return (T)destCap.getEngineItemHandler();
            } else if(managingCart.hasCapability(capability, null)) {
                return managingCart.getCapability(capability, null);
            }
        }
        return super.getCapability(capability, facing);
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public int getComparatorInputOverride(){
        if(managingCart != null) {
            if(interactEngine) {
                CapabilityMinecartDestination destCap = managingCart.getCapability(CapabilityMinecartDestination.INSTANCE, null);
                if(destCap != null && destCap.isMotorized()) {
                    return Container.func_94526_b(destCap.getFuelInv());
                } else {
                    return 0;
                }
            } else {
                int comparatorValue = 0;
                for(ICartHopperBehaviour hopperBehaviour : getApplicableHopperBehaviours(managingCart)) {
                    Capability<?> cap = hopperBehaviour.getCapability();
                    Object capabilityValue = managingCart.getCapability(cap, null);
                    if(capabilityValue != null) {
                        int behaviourComparatorValue = hopperBehaviour.getComparatorInputOverride(capabilityValue);
                        comparatorValue = Math.max(comparatorValue, behaviourComparatorValue);
                    }
                }
                return comparatorValue;
            }
        }
        return 0;
    }
}
